//=============================================================================
// File Name: main.cc
// Written by: Shang-Tse Chen (27.05.12)
//=============================================================================

#include <ctime>
#include "cmd_line.h"
#include "osboost.h"

int main(int argc, char** argv)
{
  srand(time(0));
  std::string filename;
  int weak_learner;
  int booster;
  //int dim;
  //int T;
  int N;
  double gamma;

  // parse command line argument
  learning::cmd_line cmdline;
  cmdline.info("Online SmoothBoost");
  cmdline.add("-s","input stream", &filename, "-");
  cmdline.add("-b","booster. \n     0: OSBoost \n     1: OSBoost.OCP \n     2: OSBoost.EXP \n     3: OzaBoost \n     4: OGBoost", &booster, 0);
  cmdline.add("-w","weak learner.\n     0: Perceptron \n     1: Gaussian Naive Bayes (for continuous features) \n     2: Binomial Naive Bayes (for binary features)", &weak_learner, 0);
  cmdline.add("-N","ensemble size (default: 1)", &N, 1);
  cmdline.add("-g","gamma (default: 0.1)", &gamma, 0.1);
  int rc = cmdline.parse(argc, argv);
  if (rc < 1) {
    cmdline.print_help();
    return EXIT_FAILURE;
  }


  // Construct Stream
  FileStream* stream = new FileStream(filename.c_str());

  // Set booster
  Learner* learner = NULL;
  switch (booster) {
      case 0:
          learner = new OSBoost(gamma); 
          break;
      case 1:
          learner = new OSBoostOCP(gamma); 
          break;
      case 2:
          learner = new OSBoostEXP(gamma); 
          break;
      case 3:
          learner = new OzaBoost();
          break;
      case 4:
          learner = new OGBoost();
          break;
  }

  // Set weak learners
  for(int i=0; i<N; i++){
      Learner* WL = NULL;
      switch(weak_learner){
          case 0:
              WL = new Perceptron(stream->dim);
              break;
          case 1:
              WL = new NaiveBayesGauss(stream->dim);
              break;
          case 2:
              WL = new NaiveBayesBinary(stream->dim);
              break;
      }
      WL->reset();
      if(booster <=2)
        ((OSBoost*)learner)->add_weak_learner(WL);
      else if(booster==3)
        ((OzaBoost*)learner)->add_weak_learner(WL);
      else if(booster==4)
        ((OGBoost*)learner)->add_weak_learner(WL);
  }

  stream->reset();
  learner->reset();
  Example ex; 
  double sum_loss = 0.0;
  // main loop
  while(stream->has_next()){
      // 1. got an example
      ex = stream->next(); 
      if(weak_learner == 0) {  // normalize to length 1 if using perceptron
          double snorm = ex.x.snorm();
          if(snorm>1.0)
              ex.x.scale(1.0/sqrt(snorm));
      }
      // 2. predict
      sum_loss += learner->raw_predict(ex.x) * ex.y > 0.0 ? 0.0 : 1.0;
      // 3. update learner
      learner->update(ex);
  }

  // output result
  std::cout<<"------------------------------\n";
  std::cout<<"| # of examples | error rate |\n";
  std::cout<<"------------------------------\n|";
  std::cout.width(14);
  std::cout<<stream->T<<" | ";
  std::cout.width(10);
  std::cout<<sum_loss/stream->T<<" |"<<std::endl;
  std::cout<<"------------------------------\n";
      //std::cout<<"123\n";

  return 0;
}
