// Distributed under GNU General Public License (see license.txt for details).
//
//  Copyright (c) 2007 Shai Shalev-Shwartz.
//  All Rights Reserved.
//=============================================================================
// File Name: simple_sparse_vec_hash.cc
// implements the functions of simple_sparse_vec_hash.h
// Modified by: Shang-Tse Chen (27.05.12)
//=============================================================================

//*****************************************************************************
// Included Files
//*****************************************************************************
#include "simple_sparse_vec_hash.h"

/*---------------------------------------------------------------------------*/
simple_sparse_vector::simple_sparse_vector(std::istream& is) : my_vec() {
  
  // read the number of elements
  int n = 0;
  is >> n;

  // add dummy variable x_0
  my_vec.push_back(IndexValuePair(0,1.0));
  // read the elements
  for (int i=0; i<n ; ++i) {

    // read the pair (key,val)
    uint key;
    is >> key;
    double val;
    is >> val;

    // insert to the map
    my_vec.push_back(IndexValuePair(key,val));

  }

}
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
simple_sparse_vector::simple_sparse_vector(std::istringstream& is,int n) : my_vec() {
  
  // add dummy variable x_0
  my_vec.push_back(IndexValuePair(0,1.0));
  // read the elements
  for (int i=0; i<n ; ++i) {

    // read the pair (key,val)
    uint key;
    is >> key;
    double val;
    is >> val;

    // insert to the map
    my_vec.push_back(IndexValuePair(key,val));

  }
}
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void simple_sparse_vector::scale(double s) {

  for( simple_sparse_vector_iterator it = my_vec.begin(); 
      it != my_vec.end(); it++) {
    (*it).second *= s;
  }
}
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
uint simple_sparse_vector::max_index() {

  uint d=0;
  for( simple_sparse_vector_iterator it = my_vec.begin(); 
      it != my_vec.end(); it++) {
    if ((*it).first > d) d = (*it).first;
  }
  return d;
}
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
double simple_sparse_vector::snorm() {

  double output = 0.0;
  for(simple_sparse_vector_iterator it = my_vec.begin(); 
      it != my_vec.end(); it++) {
    double tmp = (*it).second;
    output += tmp*tmp;
  }

  return(output);
}
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void simple_sparse_vector::make_binary() {

  for(simple_sparse_vector_iterator it = my_vec.begin(); 
      it != my_vec.end(); it++) {
    (*it).second = 1.0;
  }

}
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
void simple_sparse_vector::print(std::ostream& os) {

  for(simple_sparse_vector_iterator it = my_vec.begin(); 
      it != my_vec.end(); it++) {
    os << "(" << (*it).first << "," << (*it).second << ") "; 
  }
  os << std::endl;
}
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
void simple_sparse_vector::zero() {
  my_vec.clear();
}
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
double simple_sparse_vector::operator[](uint i) {
    for(simple_sparse_vector_iterator it = my_vec.begin(); 
            it != my_vec.end(); it++) {
        if(it->first == i)
            return it->second;
        else if(it->first > i)
            return 0.0;
    }
    return 0.0;
}
/*---------------------------------------------------------------------------*/

