// Distributed under GNU General Public License (see license.txt for details).
//
//  Copyright (c) 2007 Shai Shalev-Shwartz.
//  All Rights Reserved.
//==============================================================================
// File Name: simple_sparse_vec_hash.h
// Written by: Shai Shalev-Shwartz (28.01.07)
// Modified by: Shang-Tse Chen (27.05.12)
// implement a very simple hash table and sparse vector based on stl vector 
// and the modulu function
//==============================================================================

#ifndef _SHAI_SIMPLE_HASH_TABLE
#define _SHAI_SIMPLE_HASH_TABLE

//*****************************************************************************
// Included Files
//*****************************************************************************
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <sstream>


class IndexValuePair {
 public:
  IndexValuePair() { }
  IndexValuePair(uint i, double v) : first(i), second(v) { }
  uint first;
  double second;
};
 
typedef std::vector<IndexValuePair>::iterator simple_sparse_vector_iterator;


/** Implements a simple sparse vector based on stl vector.
    @author Shai Shalev-Shwartz (shais@cs.huji.ac.il)
*/
class simple_sparse_vector {
 public:

  /** Default Constructor. Allocates an all zeros sparse vector
  */
  simple_sparse_vector() {  }

  /** Constructor. Read a line from a file of pairs of string and value.
      Construct a vector from this line.
      @param is The input stream to read from.
  */
  simple_sparse_vector(std::istream& is);

  /** Constructor. Construct a vector from an istringstream.
      @param is The input stringstream to read from.
      @param n The number of elements to read
  */
  simple_sparse_vector(std::istringstream& is,int n);
  

  // Multiply a sparse vector by a scalar s
  void scale(double s);
 
  /** Returns the squared l_2 norm of the vector
      @return the squared l_2 norm of the vector
  */
  double snorm();

  /** Convert the vector to a binary vector that just indicates 
      which elements are non-zero
  */
  void make_binary();

  // return the maximal non-zero index
  uint max_index();

  // Print the content of a sparse instance to an output stream
  void print(std::ostream& os);

  // Zero all indexed elements of a sparse vector
  void zero();

  double operator[](uint i);

  std::vector<IndexValuePair> my_vec;

};


#endif
