//=============================================================================
// File Name: stream.cc
// Written by: Shang-Tse Chen (27.05.12)
//=============================================================================
#include <cstdlib>
#include <cmath>
#include <algorithm>
#include <numeric>
#include <fstream>
#include "stream.h"

FileStream::FileStream(const char* filename)
{
    // OPEN DATA FILE
    // =========================
    std::ifstream fin (filename);
    if (!fin.good()) {
        std::cerr << "error w/ " << filename << std::endl;
        exit(EXIT_FAILURE);
    }

    // Read SVM-Light data file
    // ========================
    T = 0;
    std::string line;
    while (getline(fin,line)) {
        // ignore lines which begin with #
        if (line[0] == '#') continue;
        // Erase what comes after #
        size_t pos = line.find('#');
        if (pos < line.size()) {
            line.erase(pos);
        }
        // replace ':' with white space
        int n=0;
        for (size_t pos=0; pos < line.size(); ++pos) {
            if (line[pos] == ':') {
                n++; line[pos] = ' ';
            }
        }
        // read from the string
        std::istringstream is(line);
        int label = 0;
        is >> label;
        if (label != 1 && label != -1) {
            std::cerr << "Error reading SVM-light format. Abort." << std::endl;
            exit(EXIT_FAILURE);
        }
        simple_sparse_vector instance(is,n);
        Example example(instance,label);
        data.push_back(example);
        ++T;
        int cur_max_ind = instance.max_index() + 1;
        if (cur_max_ind > dim)
            dim = cur_max_ind;
    }
    fin.close();
    t = 0;
    random_shuffle(data.begin(), data.end());
}

void FileStream::reset()
{
    t = 0;
    random_shuffle(data.begin(), data.end());
}

bool FileStream::has_next()
{
    return t<T;
}

Example& FileStream::next()
{
    return data[t++];
}

